### Name: fork
### Title: Create a new R process using the Unix 'fork' system call
### Aliases: fork
### Keywords: programming

### ** Examples


###
# Count from 1 to 10 in a separate process
###

# define the function to do the work
testfun <- function()
  {
    cat("Counting in process", getpid(), "\n")
    for(i in 1:10)
      {
        i <<- i+1  # assign into Global environment
        cat("i=",i,"\n")
      }
    cat("Done counting in process", getpid(), "\n")
  }

# run normally, the function will change our value of i
i <- 0
testfun()
i 

# Run in a separate process, our value of i remains unchanged
i <- 0
{
  pid <- fork(testfun)
  wait(pid) # wait until the child finishes, then display its exit status
}
 

###
# Use a socket to communicate between two processes.  Information
# typed on the console, which is read by the initial process, will be send
# to the child process for display. 
###
## Not run: 
##D send <- function()
##D   {
##D      pid <- getpid()
##D      con1 <- socketConnection(Sys.info()["nodename"], port = 6011)
##D      i <- 1
##D      while(TRUE)
##D        {
##D          cat("[",pid,"] ",i,": ",sep="")
##D          data <- readLines(stdin(), n=1)
##D          writeLines(data, con=con1)
##D          if( length(grep("quit", data))>0 )
##D             break;
##D          i <- i+1
##D        }
##D      close(con1)
##D   }
##D 
##D recieve <- function()
##D   {
##D      pid <- getpid()
##D      con2 <- socketConnection(port = 6011, block=TRUE, server=TRUE)
##D      i <- 1
##D      while(TRUE)
##D        {
##D           data <- readLines(con2, n=1)
##D           cat("[",pid,"] ",i,": ",sep="")
##D           writeLines(data, stdout())
##D           if( length(grep("quit", data))>0 )
##D               break;
##D           i <- i+1
##D        }
##D      close(con2)
##D   }
##D 
##D ## Important: if we aren't going to explicitly wait() for the child
##D ## process to exit, we need to set up a dummy signal handler to collect
##D ## (and then throw away) the exit status so that child processes will not
##D ## become zombies when they exit
##D handleSIGCLD()
##D 
##D # fork off the process
##D pid <- fork(recieve)
##D 
##D # start sending...
##D send()
## End(Not run)

## restore SIGCLD signal handling to the previous state
restoreSIGCLD()



